package com.loafle.overflow.container.service;

import java.util.Map;

import com.loafle.commons.rpc.RPCException;
import com.loafle.commons.rpc.registry.RPCRegistry;
import com.loafle.overflow.core.annotation.RPCService;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Component;

/**
 * Service
 */
@Component
public class Service implements InitializingBean, ApplicationContextAware {
  private static final Logger logger = LoggerFactory.getLogger(Service.class);

  private ApplicationContext applicationContext;

  @Autowired
  private RPCRegistry rpcRegistry;

  @Override
  public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
    this.applicationContext = applicationContext;
  }

  @Override
  public void afterPropertiesSet() throws Exception {
    Map<String, Object> services = this.applicationContext.getBeansWithAnnotation(RPCService.class);
    if (null == services || 0 == services.size()) {
      logger.debug("there is not service");
      return;
    }

    services.forEach((name, bean) -> {
      logger.debug("bean {}", bean.getClass().getName());
      try {
        this.rpcRegistry.registerService(bean, bean.getClass(), name);
      } catch (RPCException e) {
        logger.error("RPCRegistry", e);
      }
    });
  }
}